@@ -1,21 +1,23 @@
 /*
- * Copyright (c) 2000 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -49,7 +51,7 @@ sMethods[kIOHIDLibUserClientNumCommands] = {
 	0,
 	(IOMethod) &IOHIDLibUserClient::open,
 	kIOUCScalarIScalarO,
-	0,
+	1,
 	0
     },
     { //    kIOHIDLibUserClientClose
@@ -194,7 +196,8 @@ initWithTask(task_t owningTask, void * /* security_id */, UInt32 /* type */)
     fClient = owningTask;
     fGate = 0;
     fNubIsTerminated = false;
-        
+    fCachedOptionBits = 0;
+    
     task_reference (fClient);
     
     return true;
@@ -213,7 +216,7 @@ IOReturn IOHIDLibUserClient::clientClose(void)
         // make sure device is closed (especially on crash)
         // note radar #2729708 for a more comprehensive fix
         // probably should also subclass clientDied for crash specific code
-        fNub->close(this);
+        fNub->close(this, fCachedOptionBits);
         
         detach(fNub);
     }
@@ -224,7 +227,7 @@ IOReturn IOHIDLibUserClient::clientClose(void)
 bool IOHIDLibUserClient::start(IOService *provider)
 {
     IOWorkLoop *wl = 0;
-    
+
     if (!super::start(provider))
 	return false;
 
@@ -233,7 +236,7 @@ bool IOHIDLibUserClient::start(IOService *provider)
 	return false;
         
     fNub->retain();
-
+    
     fGate = 0;
 
     wl = getWorkLoop();
@@ -310,27 +313,42 @@ setQueueAsyncPort(OSAsyncReference asyncRef, void * vInQueue, void *, void *,
 }
 
 IOReturn IOHIDLibUserClient::
-open(void *, void *, void *, void *, void *, void *)
+open(void * flags, void *, void *, void *, void *, void *)
 {
-    IOReturn res = kIOReturnSuccess;
-
-    IOWorkLoop *wl;
-
-    if (!fNub->open(this))
-	res = kIOReturnExclusiveAccess;
+    IOReturn 		ret = kIOReturnSuccess;
+    IOOptionBits 	options = (IOOptionBits)flags;
+    
+    if (options & kIOServiceSeize)
+        do {
+            ret = clientHasPrivilege(fClient, kIOClientPrivilegeAdministrator);
+            if (ret == kIOReturnSuccess)
+                break;
+                
+            ret = clientHasPrivilege(fClient, kIOClientPrivilegeLocalUser);
+            if (ret == kIOReturnSuccess)
+                break;
+                
+            return ret;
+        } while (0);
+    
+    if (!fNub->open(this, options))
+	return kIOReturnExclusiveAccess;
         
-    return res;
+    fCachedOptionBits = options;
+
+    return kIOReturnSuccess;
 }
 
 IOReturn IOHIDLibUserClient::
 close(void *, void *, void *, void *, void *, void *gated)
 {
     if ( ! (bool) gated ) {
+    
         return fGate->runAction(closeAction);
     }
     else /* gated */ {
     
-        fNub->close(this);
+        fNub->close(this, fCachedOptionBits);
     
         // @@@ gvdl: release fWakePort leak them for the time being
     
@@ -342,8 +360,8 @@ close(void *, void *, void *, void *, void *, void *gated)
 
 bool
 IOHIDLibUserClient::didTerminate( IOService * provider, IOOptionBits options, bool * defer )
-{
-    fNub->close(this);
+{    
+    fNub->close(this, fCachedOptionBits);
     
     fNubIsTerminated = true;
     
@@ -356,6 +374,7 @@ IOHIDLibUserClient::requestTerminate( IOService * provider, IOOptionBits options
     return false;
 }
 
+
 void IOHIDLibUserClient::free()
 {
     IOWorkLoop *wl;
@@ -480,7 +499,7 @@ addElementToQueue(void * vInQueue, void * vInElementCookie,
     // add the queue to the element's queues
     if (fNub && !isInactive() && !fNubIsTerminated)
         ret = fNub->startEventDelivery (queue, elementCookie);
-            
+    
     return ret;
 }   
     // remove an element from a queue
@@ -592,7 +611,7 @@ getReport (IOHIDReportType reportType, UInt32 reportID,
             
             // make sure the element values are updated.
             if (ret == kIOReturnSuccess)
-                fNub->handleReport(mem, reportType);
+                fNub->handleReport(mem, reportType, kIOHIDReportOptionNotInterrupt);
                 
             *reportBufferSize = mem->getLength();
             mem->release();
@@ -627,7 +646,7 @@ getReportOOL(  IOHIDReportReq *reqIn,
                 
             // make sure the element values are updated.
             if (ret == kIOReturnSuccess)
-                fNub->handleReport(mem, reqIn->reportType);
+                fNub->handleReport(mem, reqIn->reportType, kIOHIDReportOptionNotInterrupt);
                 
             *sizeOut = mem->getLength();
             mem->complete();
@@ -658,7 +677,7 @@ setReport (IOHIDReportType reportType, UInt32 reportID, void *reportBuffer,
             ret = fNub->setReport(mem, reportType, reportID);
             // make sure the element values are updated.
             if (ret == kIOReturnSuccess)
-                fNub->handleReport(mem, reportType);
+                fNub->handleReport(mem, reportType, kIOHIDReportOptionNotInterrupt);
                 
             mem->release();
         }
@@ -688,7 +707,7 @@ setReportOOL (IOHIDReportReq *req, IOByteCount inCount)
             
             // make sure the element values are updated.
             if (ret == kIOReturnSuccess)
-                fNub->handleReport(mem, req->reportType);
+                fNub->handleReport(mem, req->reportType, kIOHIDReportOptionNotInterrupt);
             
             mem->complete();
             mem->release();
@@ -841,7 +860,7 @@ ReqComplete(void *obj, void *param, IOReturn res, UInt32 remaining)
         
         // make sure the element values are updated.
         if (me->fNub && !me->isInactive() && !me->fNubIsTerminated)
-            me->fNub->handleReport(pb->fMem, pb->reportType);
+            me->fNub->handleReport(pb->fMem, pb->reportType, kIOHIDReportOptionNotInterrupt);
     }
     else 
     {
