@@ -1,21 +1,22 @@
 /*
- * Copyright (c) 1998-2000 Apple Computer, Inc. All rights reserved.
- *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * The contents of this file constitute Original Code as defined in and
- * are subject to the Apple Public Source License Version 1.1 (the
- * "License").  You may not use this file except in compliance with the
- * License.  Please obtain a copy of the License at
- * http://www.apple.com/publicsource and read it before using this file.
+ * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
  * 
- * This Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
+ * 
+ * The Original Code and all software distributed under the License are
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -129,9 +130,11 @@ protected:
     task_t fClient;
     mach_port_t fWakePort;
     mach_port_t fQueuePort;
-
+    
     bool fNubIsTerminated;
-
+    
+    IOOptionBits fCachedOptionBits;
+    
     // Methods
     virtual bool
 	initWithTask(task_t owningTask, void *security_id, UInt32 type);
@@ -154,7 +157,7 @@ protected:
                                   void *, void *, void *);
 
     // Open the IOHIDDevice
-    virtual IOReturn open(void *, void *, void *,
+    virtual IOReturn open(void * flags, void *, void *,
 			  void *, void *, void *);
     
     // Close the IOHIDDevice
@@ -164,7 +167,7 @@ protected:
     virtual bool didTerminate(IOService *provider, IOOptionBits options, bool *defer);
     
     virtual bool requestTerminate( IOService * provider, IOOptionBits options );
-
+    
     virtual void free();
 
     // return the shared memory for type (called indirectly)
